/*
** Copyright (c) 2000 by Software Daten Service GesmbH
** Handelskai 94-96, 1200 Vienna,  AUSTRIA
**
** THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
** NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
** OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
** CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
** REDISTRIBUTION OF THIS SOFTWARE.
*/

package junit.extensions;

import java.util.*;
import java.io.PrintStream;

import junit.framework.*;
import junit.extensions.*;
import junit.util.*;
import junit.textui.TestRunner;

/**
 * Extension of junit.textui.TestRunner. The implementation provides the
 * following features:
 * <ul>
 * <li>repeating a test suite multiple times via command line parameters (-X n)
 * <li>executing a test suite with multiple threads simultanously (-T n)
 * <li>configurable delay between test cases (-W ms)
 * <li>verbose mode (-V)
 * </ul>
 *
 * Known limitations :
 * <ul>
 * <li>TestRunner.writer() is declared private therefore there is no way to
 *     redirect the verbose output to this variable
 * </ul>
 *
 * Examples :
 * <ul>
 * <li>junit.extensions.TestRunnerEx -X 10 ...
 * <li>junit.extensions.TestRunnerEx -X 10 -T 10 ...
 * <li>junit.extensions.TestRunnerEx -X 10 -T 10 -W 100  ...
 * <li>junit.extensions.TestRunnerEx -V ...
 * </ul>
 *
 * @author  Siegfried GOESCHL
 * @version $Id: RepositoryImpl.java,v 1.1.1.1 1999/04/23 11:43:22 wdsgoe Exp $
 */

public class TestRunnerEx extends TestRunner {

    private int nrOfRepetitions     = 0;
    private int nrOfThreads         = 0;
    private int waitTime            = 0;
    private boolean verboseMode     = false;

    /**
     * This method was created in VisualAge.
     * @param writer java.io.PrintStream
     */

    public TestRunnerEx() {
    }

    /**
     * main entry point.
     */

    public static void main(String args[]) {
        if( args.length == 0 ) {
            printUsage();
        }
        else if ( args[0].equals( "-?" ) ) {
            printUsage();
            }
        else {
            TestRunnerEx aTestRunner= new TestRunnerEx();
            aTestRunner.start(args);
        }
    }

    protected void start(String args[]) {
        for (int i= 0; i < args.length; i++) {
            if (args[i].equals("-X"))
                nrOfRepetitions= Integer.parseInt( args[i+1] );
            else if (args[i].equals("-T"))
                nrOfThreads= Integer.parseInt( args[i+1] );
            else if (args[i].equals("-W"))
                waitTime= Integer.parseInt( args[i+1] );
            else if (args[i].equals("-V"))
                verboseMode= true;

        }
        super.start( args );
    }

    protected void doRun(Test suite, boolean wait) {

        Test test = suite;

        if( nrOfRepetitions > 1 ) {
            test = new RepeatedTest( test, nrOfRepetitions );
        }

        if( nrOfThreads > 1 ) {
            test = new VeryActiveTest( test, nrOfThreads );
        }

        super.doRun( test, wait );
    }

    public void endTest(Test test) {
        if (verboseMode) {

            StringBuffer text = new StringBuffer();

            text.append( "<============" );
            text.append( Thread.currentThread().getName() );
            text.append( '@' );
            text.append( test.getClass().getName() );
            text.append( ':' );
            text.append( getTestCaseName(test) );

            System.out.println( text.toString() );
        }

        super.endTest( test );

    }

    public void startTest(Test test) {

        StringBuffer text = new StringBuffer();

        if( waitTime > 0 ) {
            try {
                Thread.currentThread().sleep(waitTime);
            }
            catch( InterruptedException e ) {}
        }

        if( verboseMode ) {

            text.append( "=============>" );
            text.append( Thread.currentThread().getName() );
            text.append( '@' );
            text.append( test.getClass().getName() );
            text.append( ':' );
            text.append( getTestCaseName(test) );

            System.out.println( text.toString() );
        }

        super.startTest( test );
    }

    private String getTestCaseName( Test test ) {
        if( test instanceof TestCase ) {
            return ((TestCase) test).name();
        }
        else {
            return "?";
        }
    }

    static void printUsage()
    {
        System.out.println("Usage: driver [-wait] [-V] [-T n] [-X m] [-C file] [-W ms] testSuiteName" );
        System.out.println("\ttestSuiteName is the name of the test suite to invoke");
        System.out.println("\tC file test data file");
        System.out.println("\tT n is number of execution threads");
        System.out.println("\tV turn verbose mode on");
        System.out.println("\tW ms is the number of milliseconds between to test case");
        System.out.println("\tX m is the number of repItitions");
        System.out.println("");
        System.out.println("System Properties :");
        System.out.println("\tjunit.data - config file path or config file for ConfigurableTest");
        System.out.println("\tjunit.argv - command line for Argv");
        System.out.println("");
        System.out.println("Examples :");
        System.out.println("\tjava -cp junit.jar junit.extensions.TestRunnerEx junit.samples.VectorTest" );
        System.out.println("\tjava -cp junit.jar junit.extensions.TestRunnerEx -T 2 -X 2 -W 10 -V junit.samples.VectorTest" );
        System.out.println("\tjava -cp junit.jar -Djunit.data=. junit.extensions.TestRunnerEx junit.tests.ConfigurableTestCaseTest");
        System.out.println("\tjava -cp junit.jar -Djunit.data=. -Djunit.argc=\"-c config/client.ini\" junit.extensions.TestRunnerEx MyTestCase");
        System.out.println("" );
    }
}