/*
** Copyright (c) 2000 by Software Daten Service GesmbH
** Handelskai 94-96, 1200 Vienna,  AUSTRIA
**
** THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
** NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
** OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
** CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
** REDISTRIBUTION OF THIS SOFTWARE.
*/

package junit.properties;

import java.io.*;
import java.util.*;

import junit.framework.TestCase;

/**
* The class stores a set of configuration parameters in the underlying
* Properties and provides a couple of helper functions for getting/setting
* commonly used datatypes.
*
* The class is not designed for thread safety since it is intended to be
* inmutable after the creation and initialization.
*
* @author  Siegfried GOESCHL
* @version $Id: RepositoryImpl.java,v 1.1.1.1 1999/04/23 11:43:22 wdsgoe Exp $
*/

public class FileTestProperties extends Properties implements TestPropertiesEx
{
    private static final String s_thisFile =
        "$Header: /junit-3/junit/properties/FileTestProperties.java 5     10.11.00 19:13 Wdsgoe $";

    /**
    * Sperator for creating a key from multiple arguments
    */

    public static final char   s_chSeperator = '.';

    /** List of directories used for searching a file */

    private String[] searchPath = null;

    public FileTestProperties()
    {
    }

    /**
    * Creates an empty property list
    *
    * @param file file to load
    */

    public FileTestProperties( String file ) throws java.io.IOException
    {
        load( null, file );
    }

    /**
    * Creates an empty property list
    *
    * @param dirs list of directories to search in
    * @param file file to load
    */

    public FileTestProperties( String[] dirs, String file ) throws java.io.IOException
    {
        load( dirs, file );
    }

    private File findMatchingFile( String[] dirs, String file )
    {
        if( file == null ) return null;

        // try to use the file name directly

        File result = new File( file );

        // try to create new file names until a hit

        if( ! result.exists() ) {
            for( int i=0; i<dirs.length; i++ ){
                result = new File( dirs[i], file );
                if( result.exists() ) break;
            }
        }

        if( ! result.exists() ) return null;
        else return result;
   }

    private void syncProperties( FileTestProperties props )
    {
        String str = null;

        Enumeration rEnum = props.propertyNames();

        while(rEnum.hasMoreElements())
        {
            str = (String) rEnum.nextElement();

            if (getProperty(str) == null) {
                this.setProperty( str, props.getProperty(str));
            }
        }
    }

    /**
    * Recursive loading
    */

    public void load( String[] dirs, String source ) throws IOException
    {
        searchPath = dirs;

        File file = findMatchingFile( searchPath, source );

        if( file == null ) throw new FileNotFoundException( source );

        InputStream inStream = new FileInputStream( file );

        super.load( inStream );

        System.out.println( "Loaded test data from file : " + file.getAbsolutePath() );

        // Iterate over all nested config files

        String[] strArr = getStrings(".default");

        for (int i=0;i<strArr.length;i++)
        {
            FileTestProperties props = new FileTestProperties(
            searchPath,
            strArr[i]
            );

            syncProperties(props);
        }
    }

    /**
    * Generates key for property file
    */

    public String makeKey( String arg1, String arg2 )
    {
        String[] args = { arg1, arg2 };
        return makeKey( args );
    }

    /**
    * Generates key for property file
    */

    public String makeKey( String arg1, String arg2, String arg3 )
    {
        String[] args = { arg1, arg2, arg3 };
        return makeKey( args );
    }

    /**
    * Generates key for property file
    */

    public String makeKey( String[] args ) {

        StringBuffer result = new StringBuffer( 64 );

        for( int i=0, lim=args.length; i<lim; i++ ) {

            result.append( args[i] );

            // only append a dot if this is not the last argument

            if( i+1 < lim ) result.append( s_chSeperator );
        }

        return result.toString();
    }

    public boolean getBoolean( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else {
            return (Boolean.valueOf( getProperty( fullKey ) )).booleanValue();
        }
    }

    public char getChar( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else {
            if( getProperty( fullKey ).length() == 0 ) return '\0';
            return getProperty( fullKey ).charAt(0);
        }
    }

    public int getInteger( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else {
            return Integer.parseInt( getProperty( fullKey ) );
        }
    }

    public String getString( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else return getProperty( fullKey );
    }

    public double getDouble( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else return Double.parseDouble( getProperty( fullKey ) );
    }

    public float getFloat( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else return Float.parseFloat( getProperty( fullKey ) );
    }

    public short getShort( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else return Short.parseShort( getProperty( fullKey ) );
    }

    public long getLong( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else return Long.parseLong( getProperty( fullKey ) );
    }

    public byte getByte( Object obj, String function, String key ) {

        String fullKey = getPropertyKey( obj, function, key );

        if( fullKey == null ) throw new IllegalArgumentException( key );
        else return Byte.parseByte( getProperty( fullKey ) );
    }

    public String[] getStrings( String key ) {

        String[] empty={};
        Object[] obarr;
        String extKey=new String();
        String defaultValue=new String();
        List prList=new LinkedList();
        int i=1;
        int j;

        extKey=key+".0";
        String resStr=getProperty(extKey,"").trim();

        if (resStr.length()==0) {
            i++;
            extKey=key+".1";
            resStr=getProperty(extKey,"").trim();
            if (resStr.length()==0) return empty;
        }

        for (;resStr.length()!=0;i++) {
            prList.add(resStr);
            extKey=key+"."+i;
            resStr=getProperty(extKey,"").trim();
        }

        obarr=prList.toArray();
        j=obarr.length;
        String[] strarr=new String[j];

        for (i=0;i<j;i++) {
            strarr[i]=(String)obarr[i];
        }

        return strarr;
    }

    /**
    * Generates the key for test data depending on the content of the
    * property file.
    *
    * @param obj invoking object
    * @param function function name or <b>null</b>
    * @param key key of entry
    *
    * @return key of the property file or <b>null</b>
    */

    private String getPropertyKey ( Object obj, String function, String param ) {

        String key = null;

        String className = obj.getClass().getName();

        // key:
        //
        // -) class name with package name
        // -) function name
        // -) parameter name

        if( function != null ) {

            key = makeKey( className, function, param );

            if( containsKey( key ) )
            {
                return key;
            }
        }

        // key:
        //
        // -) base name
        // -) function name
        // -) parameter name

        String baseName = className.substring(
            className.lastIndexOf('.')+1,
            className.length()
            );

        if( function != null ) {

            key = makeKey( baseName, function, param );

            if( containsKey( key ) )
            {
                return key;
            }
        }

        // key:
        //
        // -) base name
        // -) parameter name

        key = makeKey( baseName, param );

        if( containsKey( key ) )
        {
            return key;
        }

        // key:
        //
        // -) parameter name

        key = param;

        if( containsKey( key ) )
        {
            return key;
        }

        // this key is not defined !!

        return null;
    }

}

