/*
** Copyright (c) 2000 by Software Daten Service GesmbH
** Handelskai 94-96, 1200 Vienna,  AUSTRIA
**
** THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY OF ANY KIND,
** NOT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY. THE AUTHOR
** OF THIS SOFTWARE, ASSUMES _NO_ RESPONSIBILITY FOR ANY
** CONSEQUENCE RESULTING FROM THE USE, MODIFICATION, OR
** REDISTRIBUTION OF THIS SOFTWARE.
*/

package junit.properties;

import java.io.*;
import java.util.StringTokenizer;

import junit.framework.Test;

/**
* Factory for creating instances of classes implementing TestProperties. The
* instance is dynamically loaded using the system property
* junit.data.class
*
* @author  Siegfried GOESCHL
* @version $Id: RepositoryImpl.java,v 1.1.1.1 1999/04/23 11:43:22 wdsgoe Exp $
*/

public class TestPropertiesFactory
{
    /** Key for defining the class to load implementing TestPropertiesEx */

    public static final String JUNIT_DATA_CLASS_KEY = "junit.data.class";

    /** Key for defining the test data file */

    public static final String JUNIT_DATA_KEY = "junit.data";

    /** Extension for property file */

    public static final String JUNIT_FILE_EXTENSION = "ini";


    /** one and only instance */

    private static TestPropertiesFactory singleton = null;

    /** container for test data */

    private TestPropertiesEx testProperties = null;

    /** Return test data container */

    public TestProperties getTestProperties() {
        return testProperties;
    }

    private TestPropertiesFactory( Class testClass ) {

        try {
            load( testClass );
        }

        catch( Exception e ) {
            System.err.println( "Failed to load test data due to " + e.getMessage());
            e.printStackTrace();
            System.exit(-1);
            }
    }

    /**
     * Explicit initialization of TestPropertiesFactory in case of a master
     * test suite. Since a master test suite is usually not instantiated
     * there is no way to determine the class name and the corresponding
     * property file.
     *
     * @testClass class object of master test suite
     */

    synchronized static public void init( Class testClass ) {

        if( singleton == null ) {
            singleton = new TestPropertiesFactory( testClass );
        }
    }

    /**
     * Returns the and only instance
     */

    static public TestPropertiesFactory getSingleton() {
        return singleton;
    }

    /**
     * Tries to instantiate the file
     */

    private void load( Class testClass ) throws IOException {

        String userArg = System.getProperty( JUNIT_DATA_KEY );

        // since no argument is supplied try loading using defaults

        if( ( userArg == null ) || ( userArg.length() == 0 ) ) {

            loadUsingDirectories( testClass, System.getProperty( "java.class.path" ) );

        }
        else {

            // loading a file or using a directory as starting point ?

            File file = new File( userArg );

            if( ! file.exists() ) {
                System.out.println( "Can't load test data using " + userArg );
                System.exit(-1);
            }

            if( file.isDirectory() ) {
                loadUsingDirectories( testClass, userArg );
            }
            else {
                loadFile( file.getAbsolutePath() );
            }
        }
    }

    private void loadFile( String name ) throws IOException  {
        testProperties = createTestProperties( null, name );
    }

    private void loadUsingDirectories ( Class testClass, String dirs )
        throws IOException
    {
        String name = getTextConfigFileName( testClass.getName() );
        String[] dirList = getDirectoryList( dirs );

        testProperties = createTestProperties( dirList, name );
    }


    /**
     * Parses a list of paths seperated by ';' or ':' and returns an array
     * of directoriy names
     */

    private String[] getDirectoryList( String dirs ) throws IOException {

        int idx = 1;

        StringTokenizer tokenizer = new StringTokenizer(
            dirs,
            System.getProperty( "path.separator" )
            );

        String[] result = new String[ tokenizer.countTokens() + 1 ];

        while( tokenizer.hasMoreTokens() ) {
            result[idx++] = tokenizer.nextToken();
        }

        result[0] = System.getProperty( "user.dir" );

        return result;
    }

    /**
     * Computes the default config file name based on the class name
     */

    private String getTextConfigFileName( String className ) {

        String baseName  = className.replace( '.', '/' );
        String fileName  = baseName + '.' + JUNIT_FILE_EXTENSION;

        return fileName;
    }

    /**
     * Factory method for instantiating TestProperties
     *
     * @param dirs list of directories to start the search or <b>null</b>
     * @param file name of test property file
     */

    private TestPropertiesEx createTestProperties( String[] dirs, String file )
        throws IOException
    {
        TestPropertiesEx result = null;

        String classToLoad = System.getProperty(
            JUNIT_DATA_CLASS_KEY,
            FileTestProperties.class.getName()
            );

        try {
            Class testDataClass = Class.forName( classToLoad );
            result = (TestPropertiesEx) testDataClass.newInstance();
        }

        catch( Exception e ) {
            System.err.println(
                "Unable to instantiate TestProperties due to "
                + e.getMessage()
                );
            e.printStackTrace();
            System.exit( -1 );
        }

        try {
            result.load( dirs, file );
        }

        catch( Exception e ) {
            System.err.println(
                "Unable to load test data due to "
                + e.getMessage()
                );
            e.printStackTrace();
            System.exit( -1 );
        }

        System.out.println(
            "Using an instance of " + classToLoad +
            " to access the test data"
            );

        return result;
    }
}